/* Copyright (C) 2001 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include "samhain_monitor.h"
#include <gnome.h>
#include <applet-widget.h>

/* flag for open log window
 */
static int log_is_open = 0;       

/* the log buffer
 */
static  char log_buffer[LOG_LINES][LOG_LINE_MAX];
static  int  log_lines   = 0;
static  int  log_pos     = 0;
static  int  log_updated = 0;

/* the log window
 */
static GtkWidget   *log_window;

/* the log text
 */
static GtkWidget   *log_text;

static void close_log ( /* GtkWidget * widget, gpointer fs */)
{
  log_is_open = 0;
  gtk_widget_destroy (GTK_WIDGET (log_window));
  return;
}


static gfloat my_adj_val = -1.0;

static void update_adj (GtkAdjustment *adj)
{
  gfloat down = adj->upper - adj->page_size;
  int    diff = (my_adj_val - down);

  if (diff < 0) diff = -diff;

  /* fprintf(stderr, "%d %f %f\n", diff, down, my_adj_val); */
  if (diff <= 42)
    my_adj_val = down;

  if (my_adj_val >= 0.0)
    gtk_adjustment_set_value( adj, my_adj_val);
  else
    gtk_adjustment_set_value( adj, down);

  gtk_signal_emit_by_name (GTK_OBJECT (adj), "changed");
  return;
}

static void WriteLogText ()
{
   GdkColormap *cmap;
   GdkColor colour_red;
   GdkColor colour_orange;

#if 0
   GdkFont *fixed_font;         /* the fixed font                 */
   GdkFont *italic_font;        /* the italic font                */ 
   GdkFont *bold_font;          /* the bold font                  */
#endif
   GdkFont *normal_font;        /* the normal font                */ 

   char      * inStrg;          /* the string to put in viewer    */

   int         i;               /* standard counter variable      */
   int         pos;             /* log text position              */
   
   int bold   = 0;
   int italic = 0;
   int fixed  = 0;
   

   if (log_updated == 1)
     return;

   if (log_is_open == 0)
     return;

  cmap = gdk_colormap_get_system();
  colour_red.red   = 0xffff;
  colour_red.green = 0;
  colour_red.blue  = 0;
  if (!gdk_colormap_alloc_color(cmap, &colour_red, FALSE, TRUE)) {
    g_warning (_("Could not allocate red colour"));
  }
  colour_orange.red   = 0xffff;
  colour_orange.green = 0x7777;
  colour_orange.blue  = 0x1111;
  if (!gdk_colormap_alloc_color(cmap, &colour_orange, FALSE, TRUE)) {
    g_warning (_("Could not allocate orange colour"));
  }

   my_adj_val = (GTK_TEXT (log_text)->vadj)->value - 0.0;
   if (my_adj_val < 0.0) my_adj_val = 0.0;


  /* ------ Freeze the text widget, ready for multiple updates -- */

  gtk_text_freeze (GTK_TEXT (log_text));

  gtk_text_backward_delete(GTK_TEXT (log_text),
                           gtk_text_get_length( GTK_TEXT (log_text)));


  /* Load  fonts                                                  */
#if 0
  fixed_font  = gdk_font_load 
               ("-misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*");
  if (!fixed_font)
    g_warning (_("Could not load -misc-fixed-medium-r-*-*-*-120-*-*-*-*-*-*"));
  bold_font   = gdk_font_load 
               ("*-new century schoolbook-bold-r-normal--*-120-*-*-*-*-*-*");
  if (!bold_font)
    g_warning (_("Could not load *-new century schoolbook-bold-r-normal--*-120-*-*-*-*-*-*"));
  italic_font = gdk_font_load 
               ("*-new century schoolbook-bold-i-normal--*-120-*-*-*-*-*-*");
  if (!italic_font)
    g_warning (_("Could not load *-new century schoolbook-bold-i-normal--*-120-*-*-*-*-*-*"));
#endif
  normal_font = gdk_font_load 
               ("*-new century schoolbook-medium-r-normal--*-120-*-*-*-*-*-*");
  if (!normal_font)
    g_warning (_("Could not load *-new century schoolbook-medium-r-normal--*-120-*-*-*-*-*-*"));


  /* the first line */
  i = 0;

  while (1) {

    /* LOG_LINES is maximum, log_lines is actual number of lines
     */
    if (i >= LOG_LINES || i >= log_lines)
      break;

    if (log_lines >= LOG_LINES)
      {
	pos = i + log_pos;
    
	if (pos >= LOG_LINES)
	  pos = pos % LOG_LINES;
      }
    else 
      pos = i;

    inStrg = log_buffer[pos];
    bold   = 0;
    italic = 0;
    fixed  = 0;

    if (NULL != strstr(inStrg, _("ERROR")))
      italic = 1;
    else if (NULL != strstr(inStrg, _("CRIT")))
      bold   = 1;
    else if (NULL != strstr(inStrg, _("PANIC")))
      bold   = 1;

    if (NULL != strstr(inStrg, _("POLICY")))
      bold   = 1;
    else if (NULL != strstr(inStrg, _("PANIC")))
      bold   = 1;
    if (NULL != strstr(inStrg, _("ALERT")))
      fixed   = 1;
    
    if (bold == 1)
      {
	gtk_text_insert (GTK_TEXT (log_text), normal_font, 
			 &colour_red, NULL,
			 inStrg, -1);
	++i;
	continue;
      }
    else if (italic == 1)
      {
	gtk_text_insert (GTK_TEXT (log_text), normal_font, 
			 &colour_orange, NULL,
			 inStrg, -1);
	++i;
	continue;
      }
    else if (fixed == 1)
      {
	gtk_text_insert (GTK_TEXT (log_text), normal_font, 
			 &log_text->style->black, NULL,
			 inStrg, -1);
	++i;
	continue;
      }
    else 
      {
	gtk_text_insert (GTK_TEXT (log_text), normal_font, 
			 &log_text->style->black, NULL,
			 inStrg, -1);
	++i;
	continue;
      }
  }


  /* Thaw the text widget, allowing the updates to become visible */  

  gtk_text_thaw (GTK_TEXT (log_text));
  update_adj((GTK_TEXT (log_text)->vadj));
  log_updated = 1;

  return;

}

void insert_in_buffer (char * data)
{
  char * p;
  int    i;


  p = strtok(data, "\n");

  while (p || (p != '\0'))
    {
      if (NULL != strstr(p, "POLICY"))
	switch_to_warn(_("An policy violation has occurred"), p);
      else if (NULL != strstr(p, "ERROR"))
	switch_to_warn(_("An error has occurred"), p);
      else if (NULL != strstr(p, "CRIT"))
	switch_to_warn(_("A critical error has occurred"), p);
      else if (NULL != strstr(p, "PANIC"))
	switch_to_warn(_("A fatal error has occurred"), p);
      else if (NULL != strstr(p, "EXIT"))
	switch_to_warn(_("The daemon has exited"), p);
      else if (NULL != strstr(p, "ENOMSGQ"))
	switch_to_warn(_("No message queue (daemon not started ?)"), p);


      strncpy (log_buffer[log_pos], p, LOG_LINE_MAX - 2);
      i = strlen(log_buffer[log_pos]);
      log_buffer[log_pos][i]   = '\n';
      log_buffer[log_pos][i+1] = '\0';

      ++log_pos;
      if (log_pos >= LOG_LINES)
	log_pos = 0;

      ++log_lines;
      if (log_lines > LOG_LINES)
	log_lines = LOG_LINES;

      p = strtok(NULL, "\n");
    }
 
  /* the buffer is modified, thus the display is out of date
   */
  log_updated = 0;

  /* update the display
   */
  if (log_is_open)
    WriteLogText();

  return;
}


void log_event ( /* GtkWidget *widget, gpointer *data  */)
{

  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *button;
  GtkWidget *vscrollbar;


  /* ------- check whether already open  ----------------------   */

  if (log_is_open == 1) 
    {
      if (log_updated == 1)
	return;
      else
	{
	  WriteLogText ();
	}
      
      return;
    }

  /* ----------- open log viewer  -----------------------------   */


  log_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_usize (log_window, 580, 400);
  gtk_window_set_policy (GTK_WINDOW(log_window), TRUE, TRUE, FALSE);  
  gtk_signal_connect (GTK_OBJECT (log_window), "destroy",
                        (GtkSignalFunc) close_log, 
                         GTK_WIDGET (log_window));
  gtk_window_set_title (GTK_WINDOW (log_window), "Log");
  gtk_container_border_width (GTK_CONTAINER (log_window), 0);


  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (log_window), vbox);
  gtk_widget_show (vbox);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
  gtk_widget_show (hbox);

  /* ---------  Create the GtkText widget ----------------------  */

  log_text = gtk_text_new (NULL, NULL);
  gtk_text_set_editable (GTK_TEXT (log_text), FALSE);
  gtk_box_pack_start (GTK_BOX (hbox), log_text, TRUE, TRUE, 0);
  gtk_widget_show (log_text);
  
  /* ------- Add a vertical scrollbar to the GtkText widget ----- */

  vscrollbar = gtk_vscrollbar_new (GTK_TEXT (log_text)->vadj);
  gtk_box_pack_start(GTK_BOX(hbox), vscrollbar, FALSE, FALSE, 0);
  gtk_widget_show (vscrollbar);

  /* ----------- create buttons and add ------------------------- */

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show (hbox);

  button = gnome_stock_button(GNOME_STOCK_BUTTON_OK);

  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      (GtkSignalFunc) close_log,
                      GTK_WIDGET (log_window));
  gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
  (GTK_WIDGET_FLAGS (button)  |= (GTK_CAN_DEFAULT));
  gtk_widget_grab_default (button); 

  gtk_widget_show (button);

  /* -------- Realizing the widget to insert some text ---------- */

  gtk_widget_realize (log_text);

  /* ------------------- set flag   ----------------------------- */

  log_is_open = 1;

  /* ------------------- insert text ---------------------------- */

  WriteLogText();

  /* ------------- and show it ---------------------------------- */

  gtk_widget_show (log_window);

  return;
}

